%%% Econ 203C (Spring 2009)
%%% Problem Set VII, Question 1
%%% Monte Carlo Simulation of Bootstrap Methods

close all;              % close all the figures
clear all;              % clear all the variables
clc;                    % clear the screen

% set parameters
R = 1000;               % number of Monte Carlo simulation
n = 10;                 % sample size in each simulation
B1 = 250;               % number of Bootstrap
B2 = 1000;              % number of Bootstrap
mu = 0;                 % mean of X, unknown to the researchers
sigma2 = 6;             % varaiance of X, known to the researchers

% get intermediate parameters
theta0 = exp(mu);       % true parameter to be estimated

% Initialize the sequence of random numbers
randn('seed',2032009);
rand('seed',2032009);

for r=1:R
    % Data Generating Process
    x = randn(n,1) * sqrt(sigma2);
    
    % Estimation of theta
    theta_hat(r) = exp(mean(x));

    % Bootstrap Estimation
    u_nb = rand(n,max(B1,B2));
    x_nb = x(ceil(u_nb*n));
    theta_hat_b = exp(mean(x_nb));
    
    % Bootstrap t-statistic
    t_nb = (theta_hat_b-theta_hat(r)) ./ (theta_hat_b*sqrt(sigma2/n));
    
    % Bootstrap critical values for confidence interval
    cv_sym1(r) = quantile(abs(t_nb(1:B1)),.95); % Sym. 2-sided with B= 250
    cv_one1(r) = quantile(t_nb(1:B1),.05);      %      1-sided with B= 250
    cv_sym2(r) = quantile(abs(t_nb(1:B2)),.95); % Sym. 2-sided with B=1000
    cv_one2(r) = quantile(t_nb(1:B2),.05);      %      1-sided with B=1000
end

% Length of Confidence Intervals
length_ci_delta_sym = 2 * theta_hat*sqrt(sigma2/n) * norminv(.975);
% length_ci_delta_sym = 2 * theta_hat*sqrt(sigma2/n) * tinv(.975,n);
length_ci_boot1_sym = 2 * theta_hat*sqrt(sigma2/n) .* cv_sym1;
length_ci_boot2_sym = 2 * theta_hat*sqrt(sigma2/n) .* cv_sym2;
length_ci_delta_one = theta_hat - theta_hat*sqrt(sigma2/n) * norminv(.05);
% length_ci_delta_one = theta_hat - theta_hat*sqrt(sigma2/n) * tinv(.05,n);
length_ci_boot1_one = theta_hat - theta_hat*sqrt(sigma2/n) .* cv_one1;
length_ci_boot2_one = theta_hat - theta_hat*sqrt(sigma2/n) .* cv_one2;

% Empirical Coverage Probabilities
cov_prob_delta_sym = mean(abs(theta_hat-theta0) < length_ci_delta_sym/2);
cov_prob_boot1_sym = mean(abs(theta_hat-theta0) < length_ci_boot1_sym/2);
cov_prob_boot2_sym = mean(abs(theta_hat-theta0) < length_ci_boot2_sym/2);
cov_prob_delta_one = mean(theta0 < length_ci_delta_one);
cov_prob_boot1_one = mean(theta0 < length_ci_boot1_one);
cov_prob_boot2_one = mean(theta0 < length_ci_boot2_one);

% Average Lengths of Confidence Intervals
AvLength = [mean(length_ci_delta_sym),mean(length_ci_delta_one);
    mean(length_ci_boot1_sym),mean(length_ci_boot1_one);
    mean(length_ci_boot2_sym),mean(length_ci_boot2_one)];

% Print the result
fprintf('                      ||  Empirical Cov. Prob. |    Av. Lengths of CI\n');
fprintf('                      ||-----------------------|---------------------\n');
fprintf('                      ||     2-sided   1-sided |    2-sided   1-sided\n');
fprintf('----------------------||-----------------------|---------------------\n');
fprintf(' Delta Method         ||  %10.4f%10.4f | %10.4f%10.4f\n',cov_prob_delta_sym,cov_prob_delta_one,AvLength(1,:));
fprintf(' Bootstrap w/ B= 250  ||  %10.4f%10.4f | %10.4f%10.4f\n',cov_prob_boot1_sym,cov_prob_boot1_one,AvLength(2,:));
fprintf(' Bootstrap w/ B=1000  ||  %10.4f%10.4f | %10.4f%10.4f\n',cov_prob_boot2_sym,cov_prob_boot2_one,AvLength(3,:));
