%%% Econ 203C (Spring 2009)
%%% Problem Set VII, Question 4
%%% Monte Carlo Simulation of HAC estimator

close all;              % close all the figures
clear all;              % clear all the variables
clc;                    % clear the screen

% set parameters
R = 2000;               % number of Monte Carlo simulation
T = 100;                % sample size in each simulation
Omega = eye(2);         % variance of e and u
mu = 0;                 % true parameter of constant of AR(1)
delta = 0;              % true parameter of interest
phi = .5;               % first order parameter of x
rho = [-.5,0;...
    0,0;...
    .5,0;...
    .9,0;...
    .99,0;...
    1.5,-.75;...
    1.9,-.95;...
    .8,.1];             % nuisance parameters
x0 = 0;                 % past history of x, not used in estimation
u0 = 0;                 % past history of u, used only in data generating process
u_neg1 = 0;

% get intermediate parameters
St = floor(sqrt(T));

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the data matrix
x = zeros(T,1);
u = zeros(T,1);

% Initialize the estimates matrix
delta_est = [];
std_HAC_est = [];
std_White_est = [];

% Start iteration
for r=1:R
    % Initialize the intermediate estimates matrix
    delta_temp = [];
    std_HAC_temp = [];
    std_White_temp = [];
    
    % Generate Random Numbers
    en = randn(T,2) * chol(Omega);
    
    for i=1:size(rho,1)
        % Data Generating Process
        x(1) = phi * x0 + en(1,2);
        x(2) = phi * x(1) + en(2,2);
        u(1) = rho(i,:) * [u0;u_neg1] + en(1,1);
        u(2) = rho(i,:) * [u(1);u0] + en(2,1);
        for t=3:T
            x(t) = phi * x(t-1) + en(t,2);
            u(t) = rho(i,:) * [u(t-1);u(t-2)] + en(t,1);
        end
        y = mu + delta * x + u;
        
        % OLS estimation
        X = [ones(T,1),x];
        beta_hat = inv(X'*X)*X'*y;
        
        % HAC estimation
        u_hat = y - X*beta_hat;
        v = u_hat * ones(1,2) .* X;     % v(i,:) = x(i,:) u_hat(i)
        Omega_HAC = zeros(2,2);
        for j=0:St
            Omega_HAC = Omega_HAC + (1-abs(j)/St) * v(j+1:T,:)'*v(1:T-j,:) / T;
        end
        for j=-St:-1
            Omega_HAC = Omega_HAC + (1-abs(j)/St) * v(1:T+j,:)'*v(-j+1:T,:) / T;
        end
        var_HAC = inv(X'*X/T) * Omega_HAC * inv(X'*X/T);
        
        % White estimation
        Omega_White = v'*v/T;
        var_White = inv(X'*X/T) * Omega_White * inv(X'*X/T);
        
        % Stack estimates
        delta_temp = [delta_temp,beta_hat(2)];
        std_HAC_temp = [std_HAC_temp,sqrt(var_HAC(2,2))];
        std_White_temp = [std_White_temp,sqrt(var_White(2,2))];
    end
    
    % Stack estimates
    delta_est = [delta_est;delta_temp];
    std_HAC_est = [std_HAC_est;std_HAC_temp];
    std_White_est = [std_White_est;std_White_temp];
end

% Finite Sample Rejection Probability
rej_prob_HAC = mean(sqrt(T)*delta_est > tinv(.95,T-2)*std_HAC_est);
rej_prob_White = mean(sqrt(T)*delta_est > tinv(.95,T-2)*std_White_est);

% Print the result
results = [rho';rej_prob_HAC;rej_prob_White];
fprintf(' Parameters chosen |   Rejection Probabilities (T=%3d)\n',T);
fprintf('     rho1    rho2  |    HAC estimator  White estimator\n');
fprintf('-------------------|----------------------------------\n');
fprintf('   %6.2f  %6.2f  |  %15.4f  %15.4f\n',results);
